@@ -13,17 +13,27 @@ pub enum PublicKeyAlgorithm {
     ElgamalSign = 16,
     /// DSA (Digital Signature Algorithm)
     DSA = 17,
-    /// Elliptic Curve: RFC-6637
+    /// Elliptic Curve: RFC 9580 [formerly in RFC 6637]
     ECDH = 18,
-    /// ECDSA: RFC-6637
+    /// ECDSA: RFC 9580 [formerly in RFC 6637]
     ECDSA = 19,
     /// DEPRECATED: Elgamal (Encrypt and Sign)
     Elgamal = 20,
     /// Reserved for Diffie-Hellman (X9.42, as defined for IETF-S/MIME)
     DiffieHellman = 21,
-    /// EdDSA (not yet assigned)
-    EdDSA = 22,
-    /// Private experimental range (from OpenGPG)
+    /// EdDSA legacy format [deprecated in RFC 9580, superseded by Ed25519 (27)]
+    EdDSALegacy = 22,
+
+    /// X25519 [RFC 9580]
+    X25519 = 25,
+    /// X448 [RFC 9580]
+    X448 = 26,
+    /// Ed25519 [RFC 9580]
+    Ed25519 = 27,
+    /// Ed448 [RFC 9580]
+    Ed448 = 28,
+
+    /// Private experimental range (from OpenPGP)
     Private100 = 100,
     Private101 = 101,
     Private102 = 102,
