@@ -22,10 +22,10 @@
 #![allow(
     clippy::missing_const_for_fn,
     clippy::use_self,
-    clippy::needless_borrows_for_generic_args
+    clippy::needless_borrows_for_generic_args,
+    clippy::type_complexity,
+    clippy::incompatible_msrv
 )]
-// Enable backtraces for thiserror.
-#![cfg_attr(feature = "nightly", feature(error_generic_member_access))]
 
 #[cfg(test)]
 #[macro_use]
@@ -56,3 +56,6 @@ pub use self::packet::Signature;
 
 /// The version of this crate.
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
+
+/// Default maximum size that gets buffered.
+pub const MAX_BUFFER_SIZE: usize = 1024 * 1024 * 1024;
