@@ -1,11 +1,13 @@
-use std::fmt;
 use std::io::Read;
 
+use bitfield::bitfield;
 use bstr::{BStr, BString};
 use byteorder::{BigEndian, ByteOrder};
 use chrono::{DateTime, Duration, Utc};
 use iter_read::IterRead;
-use num_enum::{FromPrimitive, IntoPrimitive, TryFromPrimitive};
+use log::debug;
+use num_enum::{FromPrimitive, IntoPrimitive};
+use smallvec::{smallvec, SmallVec};
 
 use crate::crypto::aead::AeadAlgorithm;
 use crate::crypto::hash::HashAlgorithm;
@@ -15,48 +17,137 @@ use crate::errors::Result;
 use crate::line_writer::LineBreak;
 use crate::normalize_lines::Normalized;
 use crate::packet::signature::SignatureConfig;
-use crate::packet::PacketTrait;
+use crate::packet::{PacketTrait, SignatureVersionSpecific};
 use crate::ser::Serialize;
 use crate::types::{
-    self, CompressionAlgorithm, KeyId, KeyVersion, Mpi, PublicKeyTrait, Tag, Version,
+    self, CompressionAlgorithm, Fingerprint, KeyId, KeyVersion, PublicKeyTrait, SignatureBytes,
+    Tag, Version,
 };
-use smallvec::SmallVec;
 
 /// Signature Packet
-/// https://tools.ietf.org/html/rfc4880.html#section-5.2
-#[derive(Clone, PartialEq, Eq)]
+/// <https://www.rfc-editor.org/rfc/rfc9580.html#name-signature-packet-type-id-2>
+#[derive(Clone, PartialEq, Eq, derive_more::Debug)]
 pub struct Signature {
     packet_version: Version,
 
     pub config: SignatureConfig,
-
+    #[debug("{}", hex::encode(signed_hash_value))]
     pub signed_hash_value: [u8; 2],
-    pub signature: Vec<Mpi>,
+    pub signature: SignatureBytes,
 }
 
 impl Signature {
-    #[allow(clippy::complexity)]
-    pub fn new(
+    /// Constructor for an OpenPGP v2 signature packet.
+    /// Note: This is a historical packet version!
+    #[allow(clippy::too_many_arguments)]
+    pub fn v2(
+        packet_version: Version,
+        typ: SignatureType,
+        pub_alg: PublicKeyAlgorithm,
+        hash_alg: HashAlgorithm,
+        created: DateTime<Utc>,
+        issuer: KeyId,
+        signed_hash_value: [u8; 2],
+        signature: SignatureBytes,
+    ) -> Self {
+        Signature {
+            packet_version,
+            config: SignatureConfig {
+                typ,
+                pub_alg,
+                hash_alg,
+                hashed_subpackets: vec![],
+                unhashed_subpackets: vec![],
+                version_specific: SignatureVersionSpecific::V2 { created, issuer },
+            },
+            signed_hash_value,
+            signature,
+        }
+    }
+
+    /// Constructor for an OpenPGP v3 signature packet.
+    /// Note: This is a historical packet version!
+    #[allow(clippy::too_many_arguments)]
+    pub fn v3(
+        packet_version: Version,
+        typ: SignatureType,
+        pub_alg: PublicKeyAlgorithm,
+        hash_alg: HashAlgorithm,
+        created: DateTime<Utc>,
+        issuer: KeyId,
+        signed_hash_value: [u8; 2],
+        signature: SignatureBytes,
+    ) -> Self {
+        Signature {
+            packet_version,
+            config: SignatureConfig {
+                typ,
+                pub_alg,
+                hash_alg,
+                hashed_subpackets: vec![],
+                unhashed_subpackets: vec![],
+                version_specific: SignatureVersionSpecific::V3 { created, issuer },
+            },
+            signed_hash_value,
+            signature,
+        }
+    }
+
+    /// Constructor for an OpenPGP v4 signature packet.
+    ///
+    /// OpenPGP v4 signatures are typically used with OpenPGP v4 keys, as specified in RFC 9580
+    /// (and formerly in 4880 and 2440).
+    #[allow(clippy::too_many_arguments)]
+    pub fn v4(
         packet_version: Version,
-        version: SignatureVersion,
         typ: SignatureType,
         pub_alg: PublicKeyAlgorithm,
         hash_alg: HashAlgorithm,
         signed_hash_value: [u8; 2],
-        signature: Vec<Mpi>,
+        signature: SignatureBytes,
         hashed_subpackets: Vec<Subpacket>,
         unhashed_subpackets: Vec<Subpacket>,
     ) -> Self {
         Signature {
             packet_version,
-            config: SignatureConfig::new_v4(
-                version,
+            config: SignatureConfig {
                 typ,
                 pub_alg,
                 hash_alg,
                 hashed_subpackets,
                 unhashed_subpackets,
-            ),
+                version_specific: SignatureVersionSpecific::V4,
+            },
+            signed_hash_value,
+            signature,
+        }
+    }
+
+    /// Constructor for an OpenPGP v6 signature packet.
+    ///
+    /// OpenPGP v6 signatures are specified in RFC 9580 and only used with OpenPGP v6 keys.
+    #[allow(clippy::too_many_arguments)]
+    pub fn v6(
+        packet_version: Version,
+        typ: SignatureType,
+        pub_alg: PublicKeyAlgorithm,
+        hash_alg: HashAlgorithm,
+        signed_hash_value: [u8; 2],
+        signature: SignatureBytes,
+        hashed_subpackets: Vec<Subpacket>,
+        unhashed_subpackets: Vec<Subpacket>,
+        salt: Vec<u8>,
+    ) -> Self {
+        Signature {
+            packet_version,
+            config: SignatureConfig {
+                typ,
+                pub_alg,
+                hash_alg,
+                hashed_subpackets,
+                unhashed_subpackets,
+                version_specific: SignatureVersionSpecific::V6 { salt },
+            },
             signed_hash_value,
             signature,
         }
@@ -65,7 +156,7 @@ impl Signature {
     pub fn from_config(
         config: SignatureConfig,
         signed_hash_value: [u8; 2],
-        signature: Vec<Mpi>,
+        signature: SignatureBytes,
     ) -> Self {
         Signature {
             packet_version: Default::default(),
@@ -102,7 +193,36 @@ impl Signature {
 
         // Does any issuer or issuer fingerprint subpacket matche the identity of `sig`?
         issuers.iter().any(|&key_id| key_id == &key.key_id())
-            || issuer_fps.iter().any(|&fp| fp == key.fingerprint())
+            || issuer_fps.iter().any(|&fp| fp == &key.fingerprint())
+    }
+
+    /// Check alignment between signing key version and signature version.
+    ///
+    /// Version 6 signatures and version 6 keys are strongly linked:
+    /// - only a v6 key may produce a v6 signature
+    /// - a v6 key may only produce v6 signatures
+    fn check_signature_key_version_alignment(
+        key: &impl PublicKeyTrait,
+        config: &SignatureConfig,
+    ) -> Result<()> {
+        // Every signature made by a version 6 key MUST be a version 6 signature.
+        if key.version() == KeyVersion::V6 {
+            ensure_eq!(
+                config.version(),
+                SignatureVersion::V6,
+                "Non v6 signature by a v6 key is not allowed"
+            );
+        }
+
+        if config.version() == SignatureVersion::V6 {
+            ensure_eq!(
+                key.version(),
+                KeyVersion::V6,
+                "v6 signature by a non-v6 key is not allowed"
+            );
+        }
+
+        Ok(())
     }
 
     /// Verify this signature.
@@ -110,6 +230,8 @@ impl Signature {
     where
         R: Read,
     {
+        Self::check_signature_key_version_alignment(&key, &self.config)?;
+
         ensure!(
             Self::match_identity(self, key),
             "verify: No matching issuer or issuer_fingerprint for Key ID: {:?}",
@@ -118,6 +240,21 @@ impl Signature {
 
         let mut hasher = self.config.hash_alg.new_hasher()?;
 
+        if let SignatureVersionSpecific::V6 { salt } = &self.config.version_specific {
+            // Salt size must match the expected length for the hash algorithm that is used
+            //
+            // See: https://www.rfc-editor.org/rfc/rfc9580.html#section-5.2.3-2.10.2.1.1
+            ensure_eq!(
+                self.config.hash_alg.salt_len(),
+                Some(salt.len()),
+                "Illegal salt length {} for a V6 Signature using {:?}",
+                salt.len(),
+                self.config.hash_alg
+            );
+
+            hasher.update(salt.as_ref())
+        }
+
         if matches!(self.typ(), SignatureType::Text) {
             let normalized = Normalized::new(data.bytes().flat_map(|b| b.ok()), LineBreak::Crlf);
 
@@ -126,10 +263,21 @@ impl Signature {
         } else {
             self.config.hash_data_to_sign(&mut *hasher, data)?;
         }
-        let len = self.config.hash_signature_data(&mut *hasher)?;
+        let len = self.config.hash_signature_data(&mut hasher)?;
         hasher.update(&self.config.trailer(len)?);
 
         let hash = &hasher.finish()[..];
+
+        // Check that the high 16 bits of the hash from the signature packet match with the hash we
+        // just calculated.
+        //
+        // "When verifying a version 6 signature, an implementation MUST reject the signature if
+        // these octets do not match the first two octets of the computed hash."
+        //
+        // (See https://www.rfc-editor.org/rfc/rfc9580.html#name-notes-on-signatures)
+        //
+        // (Note: we currently also reject v4 signatures if the calculated hash doesn't match the
+        // high 16 bits in the signature packet, even though RFC 9580 doesn't strictly require this)
         ensure_eq!(
             &self.signed_hash_value,
             &hash[0..2],
@@ -160,6 +308,8 @@ impl Signature {
         let key_id = signee.key_id();
         debug!("verifying certification {:?} {:#?}", key_id, self);
 
+        Self::check_signature_key_version_alignment(&signer, &self.config)?;
+
         ensure!(
             Self::match_identity(self, signer),
             "verify_certification: No matching issuer or issuer_fingerprint for Key ID: {:?}",
@@ -168,11 +318,15 @@ impl Signature {
 
         let mut hasher = self.config.hash_alg.new_hasher()?;
 
+        if let SignatureVersionSpecific::V6 { salt } = &self.config.version_specific {
+            hasher.update(salt.as_ref())
+        }
+
         // the key of the signee
         {
             let mut key_buf = Vec::new();
             // TODO: this is different for V5
-            signee.to_writer_old(&mut key_buf)?;
+            signee.serialize_for_hashing(&mut key_buf)?;
             hasher.update(&key_buf);
         }
 
@@ -181,11 +335,11 @@ impl Signature {
             let mut packet_buf = Vec::new();
             id.to_writer(&mut packet_buf)?;
 
-            match self.config.version {
+            match self.config.version() {
                 SignatureVersion::V2 | SignatureVersion::V3 => {
                     // Nothing to do
                 }
-                SignatureVersion::V4 | SignatureVersion::V5 => {
+                SignatureVersion::V4 | SignatureVersion::V6 => {
                     let prefix = match tag {
                         Tag::UserId => 0xB4,
                         Tag::UserAttribute => 0xD1,
@@ -193,11 +347,14 @@ impl Signature {
                     };
 
                     let mut prefix_buf = [prefix, 0u8, 0u8, 0u8, 0u8];
-                    BigEndian::write_u32(&mut prefix_buf[1..], packet_buf.len() as u32);
+                    BigEndian::write_u32(&mut prefix_buf[1..], packet_buf.len().try_into()?);
 
                     // prefixes
                     hasher.update(&prefix_buf);
                 }
+                SignatureVersion::V5 => {
+                    bail!("v5 signature unsupported tpc")
+                }
                 SignatureVersion::Other(version) => {
                     bail!("unsupported signature version: {:?}", version)
                 }
@@ -206,7 +363,7 @@ impl Signature {
             hasher.update(&packet_buf);
         }
 
-        let len = self.config.hash_signature_data(&mut *hasher)?;
+        let len = self.config.hash_signature_data(&mut hasher)?;
         hasher.update(&self.config.trailer(len)?);
 
         let hash = &hasher.finish()[..];
@@ -256,8 +413,14 @@ impl Signature {
             self, signer, signee, backsig
         );
 
+        Self::check_signature_key_version_alignment(&signer, &self.config)?;
+
         let mut hasher = self.config.hash_alg.new_hasher()?;
 
+        if let SignatureVersionSpecific::V6 { salt } = &self.config.version_specific {
+            hasher.update(salt.as_ref())
+        }
+
         // Hash the two keys:
         // - for a regular binding signature, first the signer (primary), then the signee (subkey)
         // - for a "backward signature" (Primary Key Binding Signature), the order of hashing is signee (primary), signer (subkey)
@@ -266,9 +429,9 @@ impl Signature {
         {
             let mut key_buf = Vec::new();
             if !backsig {
-                signer.to_writer_old(&mut key_buf)?; // primary
+                signer.serialize_for_hashing(&mut key_buf)?; // primary
             } else {
-                signee.to_writer_old(&mut key_buf)?; // primary
+                signee.serialize_for_hashing(&mut key_buf)?; // primary
             }
 
             hasher.update(&key_buf);
@@ -277,15 +440,15 @@ impl Signature {
         {
             let mut key_buf = Vec::new();
             if !backsig {
-                signee.to_writer_old(&mut key_buf)?; // subkey
+                signee.serialize_for_hashing(&mut key_buf)?; // subkey
             } else {
-                signer.to_writer_old(&mut key_buf)?; // subkey
+                signer.serialize_for_hashing(&mut key_buf)?; // subkey
             }
 
             hasher.update(&key_buf);
         }
 
-        let len = self.config.hash_signature_data(&mut *hasher)?;
+        let len = self.config.hash_signature_data(&mut hasher)?;
         hasher.update(&self.config.trailer(len)?);
 
         let hash = &hasher.finish()[..];
@@ -302,6 +465,8 @@ impl Signature {
     pub fn verify_key(&self, key: &impl PublicKeyTrait) -> Result<()> {
         debug!("verifying key (revocation): {:#?} - {:#?}", self, key);
 
+        Self::check_signature_key_version_alignment(&key, &self.config)?;
+
         ensure!(
             Self::match_identity(self, key),
             "verify_key: No matching issuer or issuer_fingerprint for Key ID: {:?}",
@@ -310,14 +475,18 @@ impl Signature {
 
         let mut hasher = self.config.hash_alg.new_hasher()?;
 
+        if let SignatureVersionSpecific::V6 { salt } = &self.config.version_specific {
+            hasher.update(salt.as_ref())
+        }
+
         {
             let mut key_buf = Vec::new();
-            key.to_writer_old(&mut key_buf)?;
+            key.serialize_for_hashing(&mut key_buf)?;
 
             hasher.update(&key_buf);
         }
 
-        let len = self.config.hash_signature_data(&mut *hasher)?;
+        let len = self.config.hash_signature_data(&mut hasher)?;
         hasher.update(&self.config.trailer(len)?);
 
         let hash = &hasher.finish()[..];
@@ -357,7 +526,7 @@ impl Signature {
         self.config.issuer()
     }
 
-    pub fn issuer_fingerprint(&self) -> Vec<&[u8]> {
+    pub fn issuer_fingerprint(&self) -> Vec<&Fingerprint> {
         self.config.issuer_fingerprint()
     }
 
@@ -371,6 +540,16 @@ impl Signature {
             .unwrap_or_else(|| &[][..])
     }
 
+    pub fn preferred_aead_algs(&self) -> &[(SymmetricKeyAlgorithm, AeadAlgorithm)] {
+        self.config
+            .hashed_subpackets()
+            .find_map(|p| match &p.data {
+                SubpacketData::PreferredAeadAlgorithms(d) => Some(&d[..]),
+                _ => None,
+            })
+            .unwrap_or_else(|| &[][..])
+    }
+
     pub fn preferred_hash_algs(&self) -> &[HashAlgorithm] {
         self.config
             .hashed_subpackets()
@@ -545,6 +724,7 @@ pub enum SignatureVersion {
     V3 = 3,
     V4 = 4,
     V5 = 5,
+    V6 = 6,
 
     #[num_enum(catch_all)]
     Other(u8),
@@ -556,16 +736,16 @@ impl Default for SignatureVersion {
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Copy, Clone, TryFromPrimitive)]
+#[derive(Debug, PartialEq, Eq, Copy, Clone, FromPrimitive, IntoPrimitive)]
 #[repr(u8)]
 pub enum SignatureType {
     /// Signature of a binary document.
-    /// This means the signer owns it, created it, or certifies that ithas not been modified.
+    /// This means the signer owns it, created it, or certifies that it has not been modified.
     Binary = 0x00,
     /// Signature of a canonical text document.
     /// This means the signer owns it, created it, or certifies that it
     /// has not been modified.  The signature is calculated over the text
-    /// data with its line endings converted to <CR><LF>.
+    /// data with its line endings converted to `<CR><LF>`.
     Text = 0x01,
     /// Standalone signature.
     /// This signature is a signature of only its own subpacket contents.
@@ -651,6 +831,9 @@ pub enum SignatureType {
     /// party that only sees the signature, not the key or source
     /// document) that cannot include a target subpacket.
     ThirdParty = 0x50,
+
+    #[num_enum(catch_all)]
+    Other(u8),
 }
 
 #[derive(Debug, PartialEq, Eq, Copy, Clone)]
@@ -680,6 +863,10 @@ pub enum SubpacketType {
     SignatureTarget,
     EmbeddedSignature,
     IssuerFingerprint,
+    PreferredEncryptionModes, // non-RFC, may only be 1: EAX, 2: OCB
+    IntendedRecipientFingerprint,
+    // AttestedCertifications, // non-RFC
+    // KeyBlock,               // non-RFC
     PreferredAead,
     Experimental(u8),
     Other(u8),
@@ -712,7 +899,11 @@ impl SubpacketType {
             SubpacketType::SignatureTarget => 31,
             SubpacketType::EmbeddedSignature => 32,
             SubpacketType::IssuerFingerprint => 33,
-            SubpacketType::PreferredAead => 34,
+            SubpacketType::PreferredEncryptionModes => 34,
+            SubpacketType::IntendedRecipientFingerprint => 35,
+            // SubpacketType::AttestedCertifications => 37,
+            // SubpacketType::KeyBlock => 38,
+            SubpacketType::PreferredAead => 39,
             SubpacketType::Experimental(n) => *n,
             SubpacketType::Other(n) => *n,
         };
@@ -756,7 +947,11 @@ impl SubpacketType {
             31 => SubpacketType::SignatureTarget,
             32 => SubpacketType::EmbeddedSignature,
             33 => SubpacketType::IssuerFingerprint,
-            34 => SubpacketType::PreferredAead,
+            34 => SubpacketType::PreferredEncryptionModes,
+            35 => SubpacketType::IntendedRecipientFingerprint,
+            // 37 => SubpacketType::AttestedCertifications,
+            // 38 => SubpacketType::KeyBlock,
+            39 => SubpacketType::PreferredAead,
             100..=110 => SubpacketType::Experimental(n),
             _ => SubpacketType::Other(n),
         };
@@ -789,7 +984,7 @@ impl Subpacket {
     }
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(derive_more::Debug, PartialEq, Eq, Clone)]
 pub enum SubpacketData {
     /// The time the signature was made.
     SignatureCreationTime(DateTime<Utc>),
@@ -800,14 +995,15 @@ pub enum SubpacketData {
     /// The OpenPGP Key ID of the key issuing the signature.
     Issuer(KeyId),
     /// List of symmetric algorithms that indicate which algorithms the key holder prefers to use.
+    /// Renamed to "Preferred Symmetric Ciphers for v1 SEIPD" in RFC 9580
     PreferredSymmetricAlgorithms(SmallVec<[SymmetricKeyAlgorithm; 8]>),
     /// List of hash algorithms that indicate which algorithms the key holder prefers to use.
     PreferredHashAlgorithms(SmallVec<[HashAlgorithm; 8]>),
     /// List of compression algorithms that indicate which algorithms the key holder prefers to use.
     PreferredCompressionAlgorithms(SmallVec<[CompressionAlgorithm; 8]>),
-    KeyServerPreferences(SmallVec<[u8; 4]>),
-    KeyFlags(SmallVec<[u8; 1]>),
-    Features(SmallVec<[u8; 1]>),
+    KeyServerPreferences(#[debug("{}", hex::encode(_0))] SmallVec<[u8; 4]>),
+    KeyFlags(#[debug("{}", hex::encode(_0))] SmallVec<[u8; 1]>),
+    Features(#[debug("{}", hex::encode(_0))] SmallVec<[u8; 1]>),
     RevocationReason(RevocationCode, BString),
     IsPrimary(bool),
     Revocable(bool),
@@ -821,11 +1017,17 @@ pub enum SubpacketData {
     TrustSignature(u8, u8),
     RegularExpression(BString),
     ExportableCertification(bool),
-    IssuerFingerprint(KeyVersion, SmallVec<[u8; 20]>),
-    PreferredAeadAlgorithms(SmallVec<[AeadAlgorithm; 2]>),
-    Experimental(u8, SmallVec<[u8; 2]>),
-    Other(u8, Vec<u8>),
-    SignatureTarget(PublicKeyAlgorithm, HashAlgorithm, Vec<u8>),
+    IssuerFingerprint(Fingerprint),
+    PreferredEncryptionModes(SmallVec<[AeadAlgorithm; 2]>),
+    IntendedRecipientFingerprint(Fingerprint),
+    PreferredAeadAlgorithms(SmallVec<[(SymmetricKeyAlgorithm, AeadAlgorithm); 4]>),
+    Experimental(u8, #[debug("{}", hex::encode(_1))] SmallVec<[u8; 2]>),
+    Other(u8, #[debug("{}", hex::encode(_1))] Vec<u8>),
+    SignatureTarget(
+        PublicKeyAlgorithm,
+        HashAlgorithm,
+        #[debug("{}", hex::encode(_2))] Vec<u8>,
+    ),
 }
 
 bitfield! {
@@ -880,7 +1082,7 @@ pub enum RevocationCode {
     /// User ID information is no longer valid (cert revocations)
     CertUserIdInvalid = 32,
 
-    /// Private Use range (from OpenGPG)
+    /// Private Use range (from OpenPGP)
     Private100 = 100,
     Private101 = 101,
     Private102 = 102,
@@ -898,23 +1100,6 @@ pub enum RevocationCode {
     Other(u8),
 }
 
-impl fmt::Debug for Signature {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Signature")
-            .field("packet_version", &self.packet_version)
-            .field("config", &self.config)
-            .field("signed_hash_value", &hex::encode(self.signed_hash_value))
-            .field(
-                "signature",
-                &format_args!(
-                    "{:?}",
-                    self.signature.iter().map(hex::encode).collect::<Vec<_>>()
-                ),
-            )
-            .finish()
-    }
-}
-
 impl PacketTrait for Signature {
     fn packet_version(&self) -> Version {
         self.packet_version
